/*
* Component: commit
*
* Purpose:
*   Header file for Commitment implementation
*
* Last Updated     : $Author$
* Update Date Time : $Date$
* RCS File Name    : $RCSfile$
* RCS Revision     : $Revision$
* Locked by        : $Locker$
* Symbolic name    : $Name$
* State            : $State$
*/
#ifndef _COMMITLIST_H_INCLUDED_
#define _COMMITLIST_H_INCLUDED_

#include <stdio.h>

#include <dcmtk/ofstd/ofstring.h>
#include <dcmtk/ofstd/oflist.h>
#include <dcmtk/dcmdata/dctk.h>

typedef enum {
	COMMIT_STATUS_UNKNOWN = 0,
	COMMIT_STATUS_SUCCESS = 1,
	COMMIT_STATUS_FAILURE = 2
} COMMIT_STATUS;


// A general failure in processing the operation was encountered.
#define SC_REASON_PROCESSING_FAILURE                 0x0110
// No such object instance One or more of the elements in the Referenced SOP Instance Sequence was not available.
#define SC_REASON_NO_SUCH_OBJECT_INSTANCE            0x0112
// The SCP does not currently have enough resources to store the requested SOP Instance(s).
#define SC_REASON_RESOURCE_LIMITATION                0x0213
// Storage Commitment has been requested for a SOP Instance with a SOP Class that is not supported by the SCP.
#define SC_REASON_REFERENCED_SOP_CLASS_NOT_SUPPORTED 0x0122
// The SOP Class of an element in the Referenced SOP Instance Sequence did not correspond
// to the SOP class registered for this SOP Instance at the SCP.
#define SC_REASON_CLASS_INSTANCE_CONFLICT            0x0119
// The Transaction UID of the Storage Commitment Request is already in use.
#define SC_REASON_DUPLICATE_TRANSACTION_UID          0x0131


/**
* Wrapper class for commiting an object
*/
class CommitObject {
private:

	/** SOP Class UID of the object to be committed */
	OFString m_sopClassUID;
	/** SOP Instance UID of the object to be committed */
	OFString m_sopInstanceUID;
	/** commit status {unknown | success | failure} */
	COMMIT_STATUS m_status;
	/** Failure reason */
	Uint16 m_failureReason;

public:

	/**
	* Constructor for creating a new CommitObject object
	*
	* @param sopClassUID        SOP Class UID of the object to be committed
	* @param sopInstanceUID     SOP Instance UID of the object to be committed
	* @param status             commit status {unknown | success | failure}
	*/
	CommitObject(const OFString & sopClassUID, const OFString & sopInstanceUID, const COMMIT_STATUS status);

	/**
	* Constructor for creating a new CommitObject object
	*
	* @param sopClassUID        SOP Class UID of the object to be committed
	* @param sopInstanceUID     SOP Instance UID of the object to be committed
	* @param failureReason      reason of failure (commit status will be failure)
	*/
	CommitObject(const OFString & sopClassUID, const OFString & sopInstanceUID, const Uint16 failureReason);

	/**
	* Copy constructor
	*
	* @param orig   Original CommitObject object to be copied
	*/
	CommitObject(const CommitObject & orig);

	/**
	* Retrieves the SOP Class UID of the object to be committed
	*
	* @return the SOP Class UID of the object to be committed
	*/
	inline const OFString & getSopClassUID() const   { return m_sopClassUID; }

	/**
	* Retrieves the SOP Instance UID of the object to be committed
	*
	* @return the SOP Instance UID of the object to be committed
	*/
	inline const OFString & getSopInstanceUID() const { return m_sopInstanceUID; }

	/**
	* Retrieves the commit status of the object to be committed
	*
	* @return the commit status of the object to be committed
	*/
	inline const COMMIT_STATUS getStatus() const { return m_status; }

	/**
	* Retrieves the Failure Reason of the object to be committed
	*
	* @return the Failure Rerason of the object to be committed
	*/
	inline const Uint16 getFailureReason() const { return m_failureReason; }

	/**
	* Sets the commit status of this object
	*
	* @param status  commit status {unknown | success | failure}
	*/
	inline void setStatus(const COMMIT_STATUS status) { m_status = status; }

	/**
	* Prints the content of the CommitObject object to a file descriptor
	*
	* @param file   File Descriptor to print on
	*/
	void print(FILE * file) const;

	/**
	* Checks if the status of this CommitObject is known (success or failure) or unknown.
	* First checks getStatus, and if getStatus() is unknown, checks if commit flag file exists.
	* The path to the commit flag file is $COMMIT_VAR/uidflags/&lt;SOP Instance UID&gt;.
	*
	* If the commit flag file exists, the status bacame known. If the first line in commit flag file starts
	* with 'success', the status will be success, otherwise the status will be failure and the failure reason
	* will be the content of the first line.
	*
	* If the commit flag file does not exist, the status remain unknown.
	*
	* @return true   if the status after the check is known,
	*         false  otherwise (unknown)
	*/
	bool checkStatus();
};



/**
* Holds a Storage Commitment Request
*/
class CommitList {
private:
	/** Transaction UID of the Storage Commitment Request */
	OFString m_transactionUID;
	/** List of objects to be committed */
	OFList<CommitObject> m_list;
	/** ID of the device the Storage Commitment Request was received from */
	OFString m_deviceID;

	/** Indicates wether there is an uncommitted object in the object list */
	bool m_hasUnknown;

	/**
	* Builds the list of objects from the Referenced SOP Sequence of the Storage Commitment Request
	*
	* @param seq   the Referenced SOP Sequence of the Storage Commitment Request
	*
	* @throws IMException if an error occures during processing the sequence
	*/
	void processRequestList(DcmSequenceOfItems & seq);

	/**
	* Processes a Storage Commitment Request received from a DICOM device
	*
	* @param requestDataset   the Storage Commitment Request to be processed
	*
	* @throws IMException if an error occures during processing the request
	*/
	void processRequest(DcmDataset & requestDataset);

	/**
	* Loads the CommitList object from file. The name of the CommitList file is
	* $COMMIT_VAR/transactions/&lt;transaction UID&gt;
	*
	* @throws IMException if an error occures during loading the CommitList file
	*/
	void load();

	/**
	* Fills a dataset for sending Storage Commitment Result from this CommitList object.
	*
	* @param dataset   Dataset to be filled
	*
	* @return true if and only if the dataset was successfully filled
	*/
	bool fillCommitmentDataset(DcmDataset & dataset);

	/**
	* Sends Storage Commitment Result to a DICOM device
	* Starts a sendResponse task and removes the transaction file
	*
	* @throws IMException if an error occures during sending the Storage Commitment Result
	*/
	void sendCommitment();

	/**
	* Removes the transaction file
	*
	* @throws IMException if the transaction file could not be removed
	*/
	void remove();

	/**
	* Puts a task to a taskd queue
	*
	* @param cmd        command to be executed by the task
	* @param priority   task priority
	* @param queue      taskd queue where the taskd should be put
	*
	* @return true if and only if the task was successfully started
	*/
	bool startTask(const OFString & cmd, const OFString & priority, const OFString & queue);


public:
	/**
	* Constructor for creating a new CommitList object for an existing CommitList file
	*
	* @param transactionUID  Transaction UID of the Storage Commitment Request stored in the CommitList file
	*
	* @throws IMException if an error occures during loading the CommitList file
	*
	* @see #load()
	*/
	CommitList(const OFString & transactionUID);

	/**
	* Constructor for creating a new CommitList object for a Storage Commitment Request
	* received from a DICOM object.
	*
	* @param requestDataset   the Storage Commitment Request
	* @param deviceID         ID of the device the Storage Commitment Request was received from
	*
	* @throws IMException if an error occures during processing the request
	*/
	CommitList(DcmDataset & requestDataset, const OFString & deviceID);

	/**
	* Destructor for destroying the CommitList object
	*/
	~CommitList();

	/**
	* Check if all the object in the object list can be committed (there is no unknown status)
	* Sends commitment if all status is known, or if expireUnknown parameter is set. In this case send
	* failed commitment and 'Timeout reached' failure reason for objects with unknown status.
	*
	* @param expireUnknown   true means send failure commitment for objects with unknown status
	*
	* @return true if commitment was sent
	*
	* @throws IMException if an error occures
	*/
	bool commitObjects(bool expireUnknown = false);

	/**
	* Saves the CommitList into a file. The name of the CommitList file is
	* $COMMIT_VAR/transactions/&lt;transaction UID&gt;
	*
	* @throws IMException if an error occures during saving the CommitList object
	*/
	void save();
};

#endif // _COMMITLIST_H_INCLUDED_
